/* pam_hooks module */

/*
 * $Id: pam_hooks.c 13 2007-12-18 09:19:30Z zack $
 *
 * Written by Stefano Zacchiroli <zack@debian.org>
 * based on the code of Andrew Morgan's pam_permit and on the idea of
 * Enrique Ocaña's pam_preprofile
 */

#define DEFAULT_USER "nobody"
#define UNUSED

#include <stdio.h>
#include <stdlib.h>

/*
 * here, we make definitions for the externally accessible functions
 * in this file (these definitions are required for static modules
 * but strongly encouraged generally) they are used to instruct the
 * modules include file to define their prototypes.
 */

#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD

#include <security/pam_modules.h>
#include <security/_pam_macros.h>

/* tags passed to the hook script to distinguish which event has
 * triggered its invocation */
#define TAG_OPEN_SESSION	"open"
#define TAG_CLOSE_SESSION	"close"

/* --- authentication management functions --- */

PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags UNUSED,
		    int argc UNUSED, const char **argv UNUSED)
{
    int retval;
    const char *user=NULL;

    /*
     * authentication requires we know who the user wants to be
     */
    retval = pam_get_user(pamh, &user, NULL);
    if (retval != PAM_SUCCESS) {
	D(("get user returned error: %s", pam_strerror(pamh,retval)));
	return retval;
    }
    if (user == NULL || *user == '\0') {
	D(("username not known"));
	retval = pam_set_item(pamh, PAM_USER, (const void *) DEFAULT_USER);
	if (retval != PAM_SUCCESS)
	    return PAM_USER_UNKNOWN;
    }
    user = NULL;                                            /* clean up */

    return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh UNUSED, int flags UNUSED,
	       int argc UNUSED, const char **argv UNUSED)
{
     return PAM_SUCCESS;
}

/* --- account management functions --- */

PAM_EXTERN int
pam_sm_acct_mgmt(pam_handle_t *pamh UNUSED, int flags UNUSED,
		 int argc UNUSED, const char **argv UNUSED)
{
     return PAM_SUCCESS;
}

/* --- password management --- */

PAM_EXTERN int
pam_sm_chauthtok(pam_handle_t *pamh UNUSED, int flags UNUSED,
		 int argc UNUSED, const char **argv UNUSED)
{
     return PAM_SUCCESS;
}

/* --- session management --- */

static void run_pam_hook(int argc, const char **argv, const char *user,
	const char *tag)
{
    int i;
    long s;
    char *args, *argp;

    s = strlen(user) + 1;	/* alloc mem for hook args */
    s += strlen(tag) + 1;
    for (i = 0; i < argc; i++)
	s += strlen(argv[i]) + 1;
    args = malloc(s+1);

    argp = args;	/* fill hook args: real args + tag + user */
    for (i = 0; i < argc; i++)
    {
        strcpy(argp, argv[i]);
        argp += strlen(argv[i]);
	*argp ++= ' ';
    }
    strcpy(argp, tag);
    argp += strlen(tag);
    *argp ++= ' ';
    strcpy(argp, user);
    argp += strlen(user);
    *argp = '\0';

    system(args);	/* run hook */
}

PAM_EXTERN int
pam_sm_open_session(pam_handle_t *pamh UNUSED, int flags UNUSED,
		    int argc, const char **argv)
{
    const char *user = NULL;

    if (pam_get_user(pamh, &user, NULL) != PAM_SUCCESS
	    || user == NULL)
	return PAM_USER_UNKNOWN;

    run_pam_hook(argc, argv, user, TAG_OPEN_SESSION);

    return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_close_session(pam_handle_t *pamh UNUSED, int flags UNUSED,
		     int argc, const char **argv)
{
    const char *user = NULL;

    if (pam_get_user(pamh, &user, NULL) != PAM_SUCCESS
	    || user == NULL)
	return PAM_SUCCESS;

    run_pam_hook(argc, argv, user, TAG_CLOSE_SESSION);

    return PAM_SUCCESS;
}

/* end of module definition */

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_hooks_modstruct = {
    "pam_hooks",
    pam_sm_authenticate,
    pam_sm_setcred,
    pam_sm_acct_mgmt,
    pam_sm_open_session,
    pam_sm_close_session,
    pam_sm_chauthtok
};

#endif
